@@ -1,4 +1,4 @@
-﻿#ifndef XMENU_H
+#ifndef XMENU_H
 #define XMENU_H
 #include "xio.h"
 #include "config.h"
@@ -32,6 +32,9 @@ typedef char *(*tokenfunc)(struct uih_context *c);
 #define DIALOG_ONOFF 8
 #define DIALOG_COORD 9
 #define DIALOG_PALSLIDER 10
+#define DIALOG_PALPICKER 11
+#define DIALOG_IFILES 12
+#define DIALOG_ILIST 13
 
 #define DIALOGIFILE(question, filename)                                        \
     {                                                                          \
@@ -73,6 +76,10 @@ typedef char *(*tokenfunc)(struct uih_context *c);
     {                                                                          \
         question, DIALOG_SLIDER, default                                       \
     }
+#define DIALOGIFILES(question, filenames)                                      \
+    {                                                                          \
+        question, DIALOG_IFILES, 0, filenames                                  \
+    }
 
 #define DIALOGIFILE_I(_question, _filename)                                    \
     menudialogs_i18n[no_menudialogs_i18n].question = _question;                \
@@ -146,6 +153,26 @@ typedef char *(*tokenfunc)(struct uih_context *c);
     menudialogs_i18n[no_menudialogs_i18n].defint = _default;                   \
     ++no_menudialogs_i18n;
 
+#define DIALOGPALPICKER_I(_question, _default)                                 \
+    menudialogs_i18n[no_menudialogs_i18n].question = _question;                \
+    menudialogs_i18n[no_menudialogs_i18n].type = DIALOG_PALPICKER;             \
+    menudialogs_i18n[no_menudialogs_i18n].defint = _default;                   \
+    ++no_menudialogs_i18n;
+
+#define DIALOGIFILES_I(_question, _filenames)                                  \
+    menudialogs_i18n[no_menudialogs_i18n].question = _question;                \
+    menudialogs_i18n[no_menudialogs_i18n].type = DIALOG_IFILES;                \
+    menudialogs_i18n[no_menudialogs_i18n].defint = 0;                          \
+    menudialogs_i18n[no_menudialogs_i18n].defstr = _filenames;                 \
+    ++no_menudialogs_i18n;
+
+#define DIALOGILIST_I(_question, _default)                                     \
+    menudialogs_i18n[no_menudialogs_i18n].question = _question;                \
+    menudialogs_i18n[no_menudialogs_i18n].type = DIALOG_ILIST;                 \
+    menudialogs_i18n[no_menudialogs_i18n].defint = 0;                          \
+    menudialogs_i18n[no_menudialogs_i18n].defstr = _default;                   \
+    ++no_menudialogs_i18n;
+
 #define NULL_I()                                                               \
     menudialogs_i18n[no_menudialogs_i18n].question = NULL;                     \
     menudialogs_i18n[no_menudialogs_i18n].type = 0;                            \
