@@ -1,6 +1,9 @@
-﻿#include <cerrno>
+#include <cerrno>
 #include <cstring>
 #include <cstdlib>
+#include <iomanip>
+#include <QFile>
+#include <QTextStream>
 #include <QMessageBox>
 #include <QSettings>
 
@@ -13,6 +16,7 @@
 #include "play.h"
 #include "i18n.h"
 #include "xthread.h"
+#include "customdialog.h"
 
 #define LANG(name, name2)                                                      \
     MENUSTRING("lang", NULL, name, name2, 0,                                   \
@@ -62,7 +66,7 @@ const char *const uih_colornames[] = {"white", "black", "red", NULL};
  * Zoltan Kovacs <kovzol@math.u-szeged.hu>, 2003-01-05
  */
 
-#define MAX_MENUDIALOGS_I18N 104
+#define MAX_MENUDIALOGS_I18N 150
 #define Register(variable) variable = &menudialogs_i18n[no_menudialogs_i18n]
 static menudialog menudialogs_i18n[MAX_MENUDIALOGS_I18N];
 // static int no_menudialogs_i18n;
@@ -76,7 +80,9 @@ static menudialog *uih_perturbationdialog, *uih_juliadialog,
     *uih_filterdialog, *uih_shiftdialog, *uih_speeddialog, *printdialog,
     *uih_bailoutdialog, *uih_threaddialog, *saveanimdialog, *uih_juliamodedialog,
     *uih_textposdialog, *uih_fastmodedialog, *uih_timedialog, *uih_numdialog,
-    *uih_fpdialog, *palettedialog, *uih_cyclingdialog, *loadimgdialog, *palettegradientdialog
+    *uih_fpdialog, *palettedialog, *uih_cyclingdialog, *palettegradientdialog,
+    *uih_renderimgdialog, *palettepickerdialog, *loadgpldialog, *savegpldialog,
+    *uih_palettecolorsdialog
 #ifdef USE_SFFE
     ,
     *uih_sffedialog, *uih_sffeinitdialog
@@ -116,7 +122,7 @@ void uih_registermenudialogs_i18n(void)
     NULL_I();
 
     Register(uih_renderdialog);
-    DIALOGIFILE_I(TR("Dialog", "File to render:"), "fract*.xaf");
+    DIALOGIFILES_I(TR("Dialog", "Files to render:"), 0);
     DIALOGOFILE_I(TR("Dialog", "Basename:"), "anim");
     DIALOGINT_I(TR("Dialog", "Width:"), 640);
     DIALOGINT_I(TR("Dialog", "Height:"), 480);
@@ -128,6 +134,16 @@ void uih_registermenudialogs_i18n(void)
     DIALOGCHOICE_I(TR("Dialog", "Always recalculate:"), yesno, 0);
     NULL_I();
 
+    Register(uih_renderimgdialog);
+    DIALOGOFILE_I(TR("Dialog", "Basename:"), "anim");
+    DIALOGINT_I(TR("Dialog", "Width:"), 640);
+    DIALOGINT_I(TR("Dialog", "Height:"), 480);
+    DIALOGFLOAT_I(TR("Dialog", "Pixel width (cm):"), 0.025);
+    DIALOGFLOAT_I(TR("Dialog", "Pixel height (cm):"), 0.025);
+    DIALOGCHOICE_I(TR("Dialog", "Image type:"), imgtypes, 0);
+    DIALOGCHOICE_I(TR("Dialog", "Antialiasing:"), yesno, 0);
+    NULL_I();
+
     Register(uih_viewdialog);
     DIALOGCOORD_I(TR("Dialog", "Center:"), 0, 0);
     DIALOGFLOAT_I(TR("Dialog", "Radius:"), 1);
@@ -165,17 +181,13 @@ void uih_registermenudialogs_i18n(void)
     NULL_I();
 
     Register(loaddialog);
-    DIALOGIFILE_I(TR("Dialog", "Filename:"), "fract*.xpf");
+    DIALOGIFILE_I(TR("Dialog", "Filename:"), "*.png *.xpf");
     NULL_I();
 
     Register(playdialog);
     DIALOGIFILE_I(TR("Dialog", "Filename:"), "anim*.xaf");
     NULL_I();
 
-    Register(loadimgdialog);
-    DIALOGIFILE_I(TR("Dialog", "Filename:"), "fract*.png");
-    NULL_I();
-
     Register(saveimgdialog);
     DIALOGOFILE_I(TR("Dialog", "Filename:"), "fract*.png");
     NULL_I();
@@ -275,13 +287,31 @@ void uih_registermenudialogs_i18n(void)
     DIALOGPALSLIDER_I("Visualiser:", 0);
     NULL_I();
 
+    Register(uih_palettecolorsdialog);
+    for (int colidx = 0; colidx < 31; colidx++) {
+        DIALOGSTR_I(TR("Dialog", "Color:"), "000000");
+    }
+    NULL_I();
+
+    Register(palettepickerdialog);
+    DIALOGPALPICKER_I("Palette:", 0);
+    NULL_I();
+
+    Register(loadgpldialog);
+    DIALOGIFILE_I(TR("Dialog", "Load Palette Config"), "file*.gpl");
+    NULL_I();
+
+    Register(savegpldialog);
+    DIALOGOFILE_I(TR("Dialog", "Save Palette Config"), "file*.gpl");
+    NULL_I();
+
     Register(uih_cyclingdialog);
     DIALOGINT_I(TR("Dialog", "Frames per second:"), 0);
     NULL_I();
 
 #ifdef USE_SFFE
     Register(uih_sffedialog);
-    DIALOGSTR_I(TR("Dialog", "Formula:"), USER_FORMULA);
+    DIALOGILIST_I(TR("Dialog", "Formula"), USER_FORMULA);
     NULL_I();
 
     Register(uih_sffeinitdialog);
@@ -339,6 +369,11 @@ static void uih_smoothmorph(struct uih_context *c, dialogparam *p)
 
 static void uih_render(struct uih_context *c, dialogparam *d)
 {
+
+    if(fnames.size() == 0) {
+        uih_error(c, "No file Selected");
+        return;
+    }
     if (d[2].dint <= 0 || d[2].dint > 4096) {
         uih_error(
             c,
@@ -369,15 +404,82 @@ static void uih_render(struct uih_context *c, dialogparam *d)
                   "renderanim: antialiasing not supported in 256 color mode"));
         return;
     }
-    uih_renderanimation(c, d[1].dstring, (xio_path)d[0].dstring, d[2].dint,
-                        d[3].dint, d[4].number, d[5].number,
-                        (int)(1000000 / d[6].number),
-#ifdef STRUECOLOR24
-                        d[7].dint ? C256 : TRUECOLOR24,
-#else
-                        d[7].dint ? C256 : TRUECOLOR,
-#endif
-                        d[8].dint, d[9].dint, c->letterspersec, NULL);
+    for(int i=0; i < (int)fnames.size(); i++) {
+
+        QString hlpmsg = "Rendering (" + QString::number(i) + "/" +
+                QString::number(fnames.size()) + ") " + fnames[i];
+        uih_message(c, hlpmsg.toStdString().c_str());
+
+        char* curr_file = strdup(fnames[i].toStdString().c_str());
+
+        QString file_number = "_" + fnames[i].split("/").back().split(".").front() + "_";
+        char* file_suffix = strdup(file_number.toStdString().c_str());
+        char* base_name = (char *)malloc(strlen(d[1].dstring) + strlen(file_suffix) + 2);
+        strcpy(base_name, d[1].dstring);
+        strcat(base_name, file_suffix);
+
+        uih_renderanimation(c, base_name, (xio_path)curr_file, d[2].dint,
+                                d[3].dint, d[4].number, d[5].number,
+                                (int)(1000000 / d[6].number),
+        #ifdef STRUECOLOR24
+                                d[7].dint ? C256 : TRUECOLOR24,
+        #else
+                                d[7].dint ? C256 : TRUECOLOR,
+        #endif
+                                d[8].dint, d[9].dint, c->letterspersec, NULL);
+        free(base_name);
+    }
+}
+
+static void uih_renderimg(struct uih_context *c, dialogparam *d)
+{
+    xio_file f = xio_wopen(".xaos_temp.xpf");
+    if(!f) {
+        uih_error(c, "Could not Render Image");
+        return;
+    }
+
+    uih_save_position(c, f, 0);
+    xio_constpath path = ".xaos_temp.xpf";
+
+    if (d[1].dint <= 0 || d[1].dint > 4096) {
+        uih_error(
+            c,
+            TR("Error",
+               "renderanim: Width parameter must be positive integer in the range 0..4096"));
+        return;
+    }
+    if (d[2].dint <= 0 || d[2].dint > 4096) {
+        uih_error(
+            c,
+            TR("Error",
+               "renderanim: Height parameter must be positive integer in the range 0..4096"));
+        return;
+    }
+    if (d[3].number <= 0 || d[4].number <= 0) {
+        uih_error(c,
+                  TR("Error",
+                     "renderanim: Invalid real width and height dimensions"));
+        return;
+    }
+    if (d[5].dint && d[6].dint) {
+        uih_error(
+            c, TR("Error",
+                  "renderanim: antialiasing not supported in 256 color mode"));
+        return;
+    }
+
+    uih_renderanimation(c, d[0].dstring, (xio_path)path, d[1].dint,
+                            d[2].dint, d[3].number, d[4].number,
+                            (int)(1000000 / 30),
+    #ifdef STRUECOLOR24
+                            d[5].dint ? C256 : TRUECOLOR24,
+    #else
+                            d[5].dint ? C256 : TRUECOLOR,
+    #endif
+                            d[6].dint, 0, c->letterspersec, NULL);
+
+    remove(".xaos_temp.xpf");
 }
 
 static menudialog *uih_getcolordialog(struct uih_context *c)
@@ -565,6 +667,11 @@ static menudialog *uih_getpalettedialog(struct uih_context *uih)
     return (palettedialog);
 }
 
+static menudialog *uih_palettepickerdialog(struct uih_context *uih)
+{
+    return (palettepickerdialog);
+}
+
 static menudialog *uih_getpalettegradientdialog(struct uih_context *uih)
 {
     if (uih != NULL) {
@@ -623,11 +730,11 @@ static void uih_palette(struct uih_context *uih, dialogparam *p)
         uih_newimage(uih);
     }
     uih->paletteshift = shift;
+    uih->palettepickerenabled = 0;
 }
 
 static void uih_palettegradient(struct uih_context *uih, dialogparam *p)
 {
-    fflush(stdout);
     int n1 = uih->palettetype;
     int n2 = uih->paletteseed;
     int shift = uih->paletteshift;
@@ -652,6 +759,102 @@ static void uih_palettegradient(struct uih_context *uih, dialogparam *p)
         uih_newimage(uih);
     }
     uih->paletteshift = shift;
+    uih->palettepickerenabled = 0;
+}
+
+static void uih_palettecolors(struct uih_context *uih, dialogparam *p){
+    unsigned char colors[31][3];
+    memset(colors, 0, sizeof (colors));
+    for(int i=0; i < 31; i++) {
+        rgb_t color;
+        hextorgb(p[i].dstring, color);
+        colors[i][0] = color[0];
+        colors[i][1] = color[1];
+        colors[i][2] = color[2];
+    }
+    mkcustompalette(uih->palette, colors);
+    uih_newimage(uih);
+    uih->palettepickerenabled = 1;
+}
+
+static void uih_palettepicker(struct uih_context *uih, dialogparam *p)
+{
+    uih_newimage(uih);
+    uih->palettepickerenabled = 1;
+}
+
+static void uih_loadgpl(struct uih_context *uih, xio_constpath d)
+{
+    QFile *loadfile = new QFile(d);
+    unsigned char colors[31][3];
+    memset(colors, 0, sizeof (colors));
+
+    if (loadfile->open(QIODevice::ReadOnly | QIODevice::Text)) {
+        QTextStream in(loadfile);
+        QStringList colorvals= in.readAll().split("\n");
+        if((int)colorvals.size() != 36) {
+            uih_error(uih, "Corrupted palette File");
+            loadfile->close();
+            return;
+        }
+
+        for(int i = 4; i < 35; i++) {
+            QStringList currcolors = colorvals[i].split(QRegExp("\\s+"));
+            int r = currcolors[0].toInt();
+            int g = currcolors[1].toInt();
+            int b = currcolors[2].toInt();
+
+            if (r < 0 || r > 255 ||
+                g < 0 || g > 255 ||
+                b < 0 || b > 255) {
+                uih_error(uih, "RGB out of range. Failed to load palette.");
+                loadfile->close();
+                return;
+            }
+
+            colors[i-4][0] = r;
+            colors[i-4][1] = g;
+            colors[i-4][2] = b;
+        }
+        mkcustompalette(uih->palette, colors);
+        loadfile->close();
+        char s[256];
+        sprintf(s, TR("Message", "File %s opened."), d);
+        uih_message(uih, s);
+
+    } else {
+        uih_error(uih, "Failed to open palette configuration");
+        return;
+    }
+
+    uih_newimage(uih);
+    uih->palettepickerenabled = 1;
+}
+
+static void uih_savegpl(struct uih_context *uih, xio_constpath d) {
+    QFile *savefile = new QFile(d);
+    unsigned char colors[31][3];
+
+    if(savefile->open(QIODevice::WriteOnly | QIODevice::Text)) {
+            getDEFSEGMENTColor(colors);
+            QTextStream stream(savefile);
+            stream << "GIMP Palette" << "\n";
+            stream << "Name: XaoS_Palette" << "\n";
+            stream << "Columns: 16" << "\n" << "#" << "\n";
+            for(int i=0; i < 31; i++){
+                char s[256];
+                sprintf(s, "%3d %3d %3d", colors[i][0], colors[i][1], colors[i][2]);
+                stream << s << "\t color_" << QString::number(i) << "\n";
+            }
+            savefile->close();
+            char s[256];
+            sprintf(s, TR("Message", "File %s saved."), d);
+            uih_message(uih, s);
+
+        } else {
+        uih_error(uih, "Failed to save palette Configuration");
+    }
+    uih->palettepickerenabled = 1;
 }
 
 static int uih_rotateselected(struct uih_context *c, int n)
@@ -1026,6 +1229,9 @@ void uih_registermenus_i18n(void)
                  loaddialog);
     MENUDIALOG_I("file", NULL, TR("Menu", "Save"), "savepos", 0,
                  uih_saveposfile, saveposdialog);
+    SUBMENU_I("file", NULL, TR("Menu", "Save as"), "saveas");
+    MENUDIALOG_I("saveas", NULL, TR("Menu", "PNG"), "saveimg", 0,
+                 uih_savepngfile, saveimgdialog);
     MENUSEPARATOR_I("file")
     MENUDIALOGCB_I("file", NULL, TR("Menu", "Record"), "record", 0,
                    uih_saveanimfile, saveanimdialog, uih_saveanimenabled);
@@ -1033,12 +1239,10 @@ void uih_registermenus_i18n(void)
                  MENUFLAG_INTERRUPT | MENUFLAG_NOPLAY, uih_playfile,
                  playdialog);
     MENUSEPARATOR_I("file");
-    MENUDIALOG_I("file", NULL, TR("Menu", "Open image"), "loadimg",
-                MENUFLAG_INTERRUPT, uih_loadpngfile, loadimgdialog);
-    MENUDIALOG_I("file", NULL, TR("Menu", "Save image"), "saveimg", 0,
-                 uih_savepngfile, saveimgdialog);
     MENUDIALOG_I("file", NULL, TR("Menu", "Render"), "renderanim", UI,
                  uih_render, uih_renderdialog);
+    MENUDIALOG_I("file", NULL, TR("Menu", "Render Image"), "renderimg", UI,
+                 uih_renderimg, uih_renderimgdialog);
     MENUSEPARATOR_I("file");
     MENUNOP_I("file", NULL, TR("Menu", "Load random example"), "loadexample",
               MENUFLAG_INTERRUPT, uih_loadexample);
@@ -1115,6 +1319,16 @@ void uih_registermenus_i18n(void)
     MENUCDIALOG_I("palettemenu", NULL, TR("Menu", "Custom palette"), "palettegradient",
                   0, uih_palettegradient, uih_getpalettegradientdialog);
     MENUSEPARATOR_I("palettemenu");
+    MENUDIALOG_I("fractal", NULL, TR("Menu", "Palette Colors"), "palettecolors",
+                 MENUFLAG_NOMENU | MENUFLAG_INTERRUPT, uih_palettecolors,
+                 uih_palettecolorsdialog);
+    MENUCDIALOG_I("palettemenu", "x", TR("Menu", "Palette Editor"), "palettepicker",
+                  0, uih_palettepicker, uih_palettepickerdialog);
+    MENUDIALOG_I("palettemenu", NULL, TR("Menu", "Load Palette Config"), "loadgpl",
+                 0, uih_loadgpl, loadgpldialog);
+    MENUDIALOG_I("palettemenu", NULL, TR("Menu", "Save Palette Config"), "savegpl",
+                 0, uih_savegpl, savegpldialog);
+    MENUSEPARATOR_I("palettemenu");
     MENUNOPCB_I("palettemenu", "y", TR("Menu", "Color cycling"), "cycling", 0,
                 uih_cyclingsw, uih_cyclingselected);
     MENUNOPCB_I("palettemenu", "Y", TR("Menu", "Reversed color cycling"),
@@ -1456,6 +1670,13 @@ void uih_unregistermenus(void)
 #ifdef USE_SFFE
 void uih_sffein(uih_context *c, const char *text)
 {
+    // Keep only top 10 entries
+    QSettings settings;
+    QStringList values = settings.value("Formulas/UserFormulas").toStringList();
+    values.push_front(text);
+    while (values.size() > 10) values.pop_back();
+    settings.setValue("Formulas/UserFormulas", values);
+
     uih_sffeset(c, c->fcontext->userformula, text);
 }
 
