@@ -1,4 +1,4 @@
-﻿#include <QtWidgets>
+#include <QtWidgets>
 #define __USE_MINGW_ANSI_STDIO 1 // for long double support on Windows
 #include <cstdio>
 
@@ -14,6 +14,8 @@
 #include <quadmath.h>
 #endif
 
+QStringList fnames = {};
+
 QString format(number_t number)
 {
     char buf[256];
@@ -93,6 +95,25 @@ CustomDialog::CustomDialog(struct uih_context *uih, const menuitem *item,
 
             formLayout->addRow(label, layout);
 
+        } else if (dialog[i].type == DIALOG_IFILES) {
+
+            QTextEdit *filenames = new QTextEdit(this);
+            filenames->setMaximumHeight(filenames->height() * 2);
+            filenames->setObjectName(label);
+
+            QToolButton *chooser = new QToolButton(this);
+            chooser->setObjectName(label);
+            chooser->setText("...");
+            connect(chooser, SIGNAL(clicked()), this,
+                    SLOT(chooseInputFiles()));
+
+            QBoxLayout *layout = new QBoxLayout(QBoxLayout::LeftToRight);
+            layout->setContentsMargins(0, 0, 0, 0);
+            layout->addWidget(filenames);
+            layout->addWidget(chooser);
+
+            formLayout->addRow(label, layout);
+
         } else if (dialog[i].type == DIALOG_CHOICE) {
 
             QComboBox *combo = new QComboBox(this);
@@ -105,7 +126,7 @@ CustomDialog::CustomDialog(struct uih_context *uih, const menuitem *item,
 
             formLayout->addRow(label, combo);
 
-        } else if(dialog[i].type == DIALOG_PALSLIDER) {
+        } else if (dialog[i].type == DIALOG_PALSLIDER) {
 
             gradientpal = clonepalette(uih->image->palette);
             palcontext = uih;
@@ -168,6 +189,52 @@ CustomDialog::CustomDialog(struct uih_context *uih, const menuitem *item,
             connect(shiftno,SIGNAL(valueChanged(int)), shiftslider, SLOT(setValue(int)));
             connect(shiftslider, SIGNAL(valueChanged(int)), shiftno, SLOT(setValue(int)));
             connect(shiftno, SIGNAL(valueChanged(int)), this, SLOT(updateVisualiser()));
+
+        } else if (dialog[i].type == DIALOG_PALPICKER) {
+
+            palcontext = uih;
+            getDEFSEGMENTColor(newColors);
+
+            QList< QPushButton* > buttons;
+            QBoxLayout *layout1 = new QBoxLayout(QBoxLayout::LeftToRight);
+            QBoxLayout *layout2 = new QBoxLayout(QBoxLayout::LeftToRight);
+            QBoxLayout *layout3 = new QBoxLayout(QBoxLayout::LeftToRight);
+            for(auto bidx = 0; bidx < 31; ++bidx ) {
+                auto button = new QPushButton{ QString::number(bidx) };
+                button->setObjectName(QString::number(bidx));
+                QColor color(newColors[bidx][0], newColors[bidx][1], newColors[bidx][2]);
+                QPalette pal = button->palette();
+                button->setAutoFillBackground(true);
+                pal.setColor(QPalette::Button, color);
+                button->setPalette(pal);
+                button->update();
+                buttons << button;
+                if(bidx <= 10)
+                    layout1->addWidget(button);
+                else if(bidx>10 and bidx <= 20)
+                    layout2->addWidget(button);
+                else
+                    layout3->addWidget(button);
+
+                connect(button, SIGNAL(clicked()), this, SLOT(colorPicker()));
+            }
+            formLayout->addRow(layout1);
+            formLayout->addRow(layout2);
+            formLayout->addRow(layout3);
+
+        } else if (dialog[i].type == DIALOG_ILIST) {
+
+            QComboBox *list = new QComboBox(this);
+            list->setObjectName(label);
+            list->setEditable(true);
+            list->addItem(dialog[i].defstr);
+
+            QSettings settings;
+            QStringList formulas = settings.value("Formulas/UserFormulas").toStringList();
+            list->addItems(formulas);
+
+            formLayout->addRow(label, list);
+
         } else {
 
             QLineEdit *field = new QLineEdit(this);
@@ -224,6 +291,13 @@ void CustomDialog::accept()
             QComboBox *field = findChild<QComboBox *>(label);
             m_parameters[i].dint = field->currentIndex();
 
+        }
+        else if (m_dialog[i].type == DIALOG_IFILES){
+
+            QTextEdit *field = findChild<QTextEdit *>(label);
+            QString raw_fnames = field->toPlainText();
+            fnames = raw_fnames.split("\n");
+
         } else {
 
             QLineEdit *field = findChild<QLineEdit *>(label);
@@ -240,6 +314,11 @@ void CustomDialog::accept()
                 palcontext->paletteshift = shiftno->value();
                 m_parameters[i].dint = 1;
                 destroypalette(gradientpal);
+            } else if (m_dialog[i].type == DIALOG_PALPICKER) {
+                mkcustompalette(palcontext->image->palette, newColors);
+            } else if (m_dialog[i].type == DIALOG_ILIST) {
+                QComboBox *list = findChild<QComboBox *>(label);
+                m_parameters[i].dstring = strdup(list->currentText().toUtf8());
             }
             else
                 m_parameters[i].dstring = strdup(field->text().toUtf8());
@@ -258,7 +337,7 @@ void CustomDialog::chooseInputFile()
     QSettings settings;
     QString fileLocation = settings.value("MainWindow/lastFileLocation", QDir::homePath()).toString();
     QString fileName = QFileDialog::getOpenFileName(
-        this, sender()->objectName(), fileLocation, "*.xpf *.xaf");
+            this, sender()->objectName(), fileLocation, "*.xpf *.png *.xaf");
     if (!fileName.isNull()) {
         field->setText(fileName);
         settings.setValue("MainWindow/lastFileLocation", QFileInfo(fileName).absolutePath());
@@ -278,6 +357,21 @@ void CustomDialog::chooseOutputFile()
     }
 }
 
+void CustomDialog::chooseInputFiles()
+{
+    QTextEdit *field = findChild<QTextEdit *>(sender()->objectName());
+    QSettings settings;
+    QString fileLocation = settings.value("MainWindow/lastFileLocation", QDir::homePath()).toString();
+    QStringList fileNames = QFileDialog::getOpenFileNames(
+        this, sender()->objectName(), fileLocation, "*.xpf *.xaf");
+    if(!fileNames.isEmpty()) {
+        for(auto file: fileNames) {
+            field->append(file);
+        }
+        settings.setValue("MainWindow/lastFileLocation", QFileInfo(fileNames[0]).absolutePath());
+    }
+}
+
 void CustomDialog::updateVisualiser()
 {
     // Get updated Colors
@@ -295,6 +389,23 @@ void CustomDialog::updateVisualiser()
     }
 
     // Save Result
-    QPixmap newImage = QPixmap::fromImage(palImage.scaled(this->algono->width(), this->algono->height()));
+    QPixmap newImage = QPixmap::fromImage(palImage.scaled(this->algono->width(),
+                                                          this->algono->height()));
     img->setPixmap(newImage);
 }
+
+void CustomDialog::colorPicker()
+{
+    QPushButton* button = qobject_cast<QPushButton*>(sender());
+    int idx = button->objectName().toInt();
+    QColor color = QColorDialog::getColor(QColor(newColors[idx][0], newColors[idx][1],
+            newColors[idx][2]), this);
+    QPalette pal = button->palette();
+    button->setAutoFillBackground(true);
+    pal.setColor(QPalette::Button, color);
+    button->setPalette(pal);
+    button->update();
+    newColors[idx][0] = color.red();
+    newColors[idx][1] = color.green();
+    newColors[idx][2] = color.blue();
+}
